% MXL-MLE
% Estimate mixed logit model using maximum likelihood method
% Code by Norman Shiau
clear all
format compact
format short
clc

global XR XF NR NF XRTYPE ND NJ 
global draws sales 
%diary output.txt

PARAMTOL=1e-8;
LLTOL=1e-5;
%DRAWS=randn(NR,ND); save normal_draws.mat DRAWS; % creating new draws
draws=0; load draws.mat; % if draws were created, read them from file
ND=length(draws); % # of random draws

% structure of DATA matrix
DATA=csvread('auto07-new.csv');
NJ=size(DATA,1); % # of cars models in the market (# of alternatives)
idmake=DATA(:,1);
idmodel=DATA(:,2);
twoseater=DATA(:,3);
subcompact=DATA(:,4);
compact=DATA(:,5);
midsize=DATA(:,6);
large=DATA(:,7);
hybrid=DATA(:,8);
luxury=DATA(:,9);
sports=DATA(:,10);
import=DATA(:,11);
hp=DATA(:,12);
weight=DATA(:,13);
citympg=DATA(:,14);
hiwympg=DATA(:,15);
vehlength=DATA(:,16);
vehwidth=DATA(:,17);
msrp=DATA(:,18);
sales=DATA(:,19);
%gasprice=2.62; % 2006 average gasoline price
mpg=1./(0.55./citympg+0.45./hiwympg);
gasprice=2.85; % 2007 average gasoline price
dpm=gasprice./mpg; % dollar per mile (dpm) in cent
hpdwt=hp./weight; % horsepower/pound
%Q=7588918;
Q=sum(sales,1);
S=sales/Q; % observed market shares
%q0=Q-sum(sales);
footprint=vehlength.*vehwidth/144; % unit: sq-feet
%%%%%%%%%%%%%%% end of pre-processing data

% attributes for   mean coefficients "beta"
XF=[twoseater,subcompact,compact,large,luxury,sports,import,hybrid];
NF=size(XF,2);
% scaling the variables for estimation
XR=[msrp/10000,dpm*100,hpdwt*100,footprint/100];
NR=size(XR,2);
draws=draws(1:NR,:);

%%%%%%%%%%%%%%% end of pre-processing data
XFNAME={'two-seater';'subcompact'; 'compact'; 'large'; 'luxury'; 'sports'; 'import'; 'hybrid'};
XRNAME={'price'; 'dpm'; 'hpdwt'; 'footprint'};
% set distribution type for sigma 1: normal, 2: log-normal
XRTYPE=[1;1;1;1];
% do a double check on the variables settings
if (NF~=length(XFNAME)), disp('Fixed coefficient settings are wrong.'); break; end
if (NR~=length(XRNAME)) || (NR~=length(XRTYPE)) || (length(XRTYPE)~=length(XRNAME)),
    disp('Random coefficient settings are wrong.'); break; end

% starting values for with outside good
parF0=zeros(NF,1);
parR0=[0;0;0;0; .1;.1;.1;.1];
par0=[ parF0; parR0];

options = optimset('Display','iter','TolX',PARAMTOL,'TolFun',LLTOL,...
        'MaxFunEvals',10000,'LargeScale','off'); %,'Algorithm','active-set');
tic;
[parhat,nfval,exitflag,output,grad,hessian] = fminunc(@LLmxl,par0,options);
%[parhat,fval,exitflag] = ktrlink(@LLmxl,par0,[],[],[],[],[],[],[],options);
toc;

% display estimation results
disp(['fminunc exitflag = ' num2str(exitflag)]);
if exitflag == 1
    disp('** Convergence achieved **');
elseif exitflag == 2
    disp('Convergence achieved by criterion based on change in parameters.');
    disp(['Parameters changed less than PARAMTOL= ' num2str(PARAMTOL)]);
    disp('You might want to check whether this is actually convergence.');
elseif exitflag == 3
    disp('Convergence achieved by criterion based on change in log-likelihood value.');
    disp(['Log-likelihood value changed less than LLTOL= ' num2str(LLTOL)]);
    disp('You might want to check whether this is actually convergence.');
else
    disp('** Convergence not achieved **');
    disp('Results are not printed because no convergence.');
    break
end


%%%%%%%%%%%%%% post data processing and display results on screen

parF=parhat(1:NF);  % estimates for fixed coefficients
parB=parhat(NF+1:NF+NR); % estimates for mean of random coefficients
parW=parhat(NF+NR+1:NF+NR*2);  % estimates for std-dev of random coefficients
r=repmat(parB,[1,ND])+draws.*repmat(parW,[1,ND]);% normal-dist betas (1)
r(XRTYPE(:,1)==2,:)=exp(r(XRTYPE(:,1)==2,:)); % for lognormal terms (2)
sj=gets(parhat);

% Taking inverse of hessian for standard errors
ihess=inv(hessian);
se=sqrt(diag(ihess)); % standard errors
%Calculate standard errors of parameters
seF=se(1:NF);
seB=se(NF+1:NF+NR);
seW=se(NF+NR+1:NF+NR*2);
LL=-nfval;
tstat=(parhat-0)./se;
LL0=sum(sales.*log(1/NJ),1);
MR2=1-LL/LL0;

disp('---------------------------------------------------------------------------------------');
disp('     Attribute |     Type  | mean_est  Std.Err |  SD_est  Std.Err | Beta_mean Beta_SD');
disp('---------------------------------------------------------------------------------------');
TYPE={'Normal'; 'Lognormal'};
beta=[mean(r,2) std(r,0,2)];
for i=1:NR,
    mm=XRTYPE(i,1);
    disp(sprintf('%14s |%10s |%8.4f %8.4f  |%8.4f %8.4f |%8.4f %8.4f',...
       XRNAME{i,1},TYPE{mm,1},[parB(i,1) seB(i,1) parW(i,1) seW(i,1) beta(i,1) beta(i,2)]));
end
for i=1:NF,
    disp(sprintf('%14s |%10s |%8.4f %8.4f  |',...
        XFNAME{i,1}, 'Fixed',  [parF(i,1) seF(i,1)]));
end
disp('--------------------------------------------------------------------------------------');
disp(sprintf('No outside good utility '));
disp(sprintf('Log-Likelihood = %-0.4e',LL));
disp(sprintf('           LL0 = %-0.4e',LL0));
disp(sprintf('  McFadden R^2 = %-0.4f',MR2));
%disp(sprintf('grad*inv(hessian)*grad = %-0.4e',grad'*ihess*grad));
disp('--------------------------------------------------------------------------------------');

% calculate demand elasticities to price and vehicle attributes
% considering all regular domestic midsize cars
elasregmid=zeros(1,NR);
sregmid=sum(sj.*midsize.*(1-luxury).*(1-sports).*(1-hybrid));
for i=1:NR, % loop over price, dpm, hp/weight, footprint
    vF=repmat(XF*parF,[1,ND]);
    XRnew=XR;
    XRnew(:,i)=XRnew(:,i)+XR(:,i).*midsize.*(1-luxury).*(1-sports).*(1-hybrid)*0.01; % add 1% increase
    vRnew=sum(repmat(XRnew,[1,1,ND]).*repmat(reshape(r,[1,NR,ND]),[NJ,1,1]),2);
    vnew=vF+permute(vRnew,[1,3,2]);
    evnew=exp(vnew); clear vF vR vnew;
    sumevnew=sum(evnew,1);
    sjdnew=evnew./repmat(sumevnew,[NJ,1]);
    sjnew=mean(sjdnew,2);
    sregmidnew=sum(sjnew.*midsize.*(1-luxury).*(1-sports).*(1-hybrid));
    elasregmid(1,i)=(sregmidnew-sregmid)/sregmid/0.01;
end

disp(' ');
disp('Demand elasticities of regular mid-size vehicle attributes :');
disp('--------------------------------------------------------------------------------------');
disp('     Attribute |   Elasticity');
disp('--------------------------------------------------------------------------------------');
for i=1:NR,
    disp(sprintf('%14s |  %8.4f ',...
       XRNAME{i,1},elasregmid(i)));
end
disp('--------------------------------------------------------------------------------------');


% elasticity matrix with 1% increase in price
ELASNAME={'Two-Seater'; 'Subcompact'; 'Compact'; 'Mid-size'; 'Large';
          'Luxury'; 'Sports'; 'Hybrid' };
pp(:,1)=msrp+twoseater.*msrp*0.01; 
pp(:,2)=msrp+subcompact.*msrp*0.01;
pp(:,3)=msrp+compact.*msrp*0.01;
pp(:,4)=msrp+midsize.*msrp*0.01;
pp(:,5)=msrp+large.*msrp*0.01;
pp(:,6)=msrp+luxury.*msrp*0.01;
pp(:,7)=msrp+sports.*msrp*0.01;
pp(:,8)=msrp+hybrid.*msrp*0.01;
dep=zeros(8,8);
for i=1:8, % loop over all classes for demand elasticities of price
    vF=repmat(XF*parF,[1,ND]);
    XRnew=[pp(:,i)/1e4,XR(:,2:end)];
    vRnew=sum(repmat(XRnew,[1,1,ND]).*repmat(reshape(r,[1,NR,ND]),[NJ,1,1]),2);
    vnew=vF+permute(vRnew,[1,3,2]);
    evnew=exp(vnew); clear vF vR vnew;
    sumevnew=sum(evnew,1);
    sjdnew=evnew./repmat(sumevnew,[NJ,1]);
    sjnew=mean(sjdnew,2);
    dep(i,1)=(sum(sjnew(twoseater==1))-sum(sj(twoseater==1)))/sum(sj(twoseater==1))/0.01;
    dep(i,2)=(sum(sjnew(subcompact==1))-sum(sj(subcompact==1)))/sum(sj(subcompact==1))/0.01;
    dep(i,3)=(sum(sjnew(compact==1))-sum(sj(compact==1)))/sum(sj(compact==1))/0.01;
    dep(i,4)=(sum(sjnew(midsize==1))-sum(sj(midsize==1)))/sum(sj(midsize==1))/0.01;
    dep(i,5)=(sum(sjnew(large==1))-sum(sj(large==1)))/sum(sj(large==1))/0.01;
    dep(i,6)=(sum(sjnew(luxury==1))-sum(sj(luxury==1)))/sum(sj(luxury==1))/0.01;
    dep(i,7)=(sum(sjnew(sports==1))-sum(sj(sports==1)))/sum(sj(sports==1))/0.01;
    dep(i,8)=(sum(sjnew(hybrid==1))-sum(sj(hybrid==1)))/sum(sj(hybrid==1))/0.01;
end

disp(' ');
disp('Increase all prices in the vehicle class for 1% and calculate demand elastisties');
disp('--------------------------------------------------------------------------------------');
disp('     Attribute |   1       2       3       4       5       6       7       8');  
disp('--------------------------------------------------------------------------------------');
for i=1:NF,
    disp(sprintf('%1d%13s | %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f',...
       i,ELASNAME{i,1},dep(i,1),dep(i,2),dep(i,3),dep(i,4),dep(i,5),dep(i,6),dep(i,7),dep(i,8) ));
end
disp('--------------------------------------------------------------------------------------');

% save solutions to file
%save solutions.mat parhat draws dep elasregmid

%diary off
% [EOF]