% Use conditional FOC algorithm to solve multi-vehicle design responses
%
% under CAFE regulations (6 cases: updated 2/23/2009)
% ---
%  demand model: New 2007 auto demand model using Ward's Auto 2007 data
% vehicle model: ADVISOR mid-size vehicle model
%    cost model: JMD 2004 exponential cost model
% ---
% Assumption: each firm has signle generic vehicle model
format compact
clc
clear all
global beta sigma
global aF aH bM c0 b1 b2 b3 b4
global J K ND Q cI
global gasprice rho mpgstd

%%%%%%%%%% set up parameters %%%%%%%%%%%%%
K=10;  % # of producers (for calculating price only)
J=1;  % # of vehicle for each firm
gasprice=2.5; % base gas price = $2.5/gal
rho=55*1e-4;  % CAFE penalty parameter in 10000 dollars
mpgstd=27.5;  % CAFE mpg standard parameter (kappa in the paper)
% calcaulate DPM (dollar per mile) using 2007 average gas price 
load draws.mat % read saved normal random draws (draws)
draws=draws(:,1:3);
ND=length(draws);
Q=3; % mid-size market size 3 million
cI=550; % investment cost = $550 million

% Meta-model from ADVISOR-2004: mid-size body with 95kW engine TX_AUTO4_4L60E
bM=95;
aF(3)= 4.8978; aF(2)=-24.74; aF(1)=48.83;  % coefficients for MPG
aH(3)=-0.4430; aH(2)=3.8733; aH(1)=0.1236; % coefficients for hp/100LB

% cost coefficients from Michalek et al. JMD (2004)
% scaled from 2004 to 2007 dollars using Consumer Price Index
c0=7500/1.0724*1.1204;
b1=670.51/1.0724*1.1204;
b2=0.0063;
%
% coefficient of my technology-cost curve (from NHTSA 2008 report)
% the costs have been converted from 2006 to 2007 dollars
b3=85936; b4=-2177; % high
%%%%b3=34121; b4=-847; % mean
%%%%b3=16699; b4=-639; % low
%
% 6/16/2009: new mixed logit model (normal distribution) 
% price/10000, dpm*100, HP/weight*100
% all normal distribution
beta =[-0.9108 -0.1812  0.2417];
sigma=[ 0.4683  0.1447  0.0042];
%
%%%%%%%%% end of setting up parameters %%%%%%%%%%%%%%%%%%%%%
disp(sprintf('Gas-Price_gamma=%-0.2f Penalty_rho=%-0.2f CAFE-std_kappa=%-0.2f',gasprice,rho*1e4,mpgstd));
options = optimset('Display','Off','TolX',1e-7,'TolFun',1e-7,'TolCon',1e-7, ...
                   'LargeScale','Off','Algorithm','active-set');
% giving upper/lower bounds
LB(:,:,1)=zeros(K,J,1);     UB(:,:,1)=ones(K,J,1)*5;   % price/10000 (0-5)
LB(:,:,2)= ones(K,J,1)*.8;  UB(:,:,2)=ones(K,J,1)*2;   % engine size XE (0.8-2.0)
LB(:,:,3)=zeros(K,J,1);     UB(:,:,3)=ones(K,J,1)*0.2; % tech-variable XT (0-0.2)
LB(:,:,4)=0; UB(:,:,4)=Inf; % Lagrangian multiplier for XE lower bound
LB(:,:,5)=0; UB(:,:,5)=Inf; % Lagrangian multiplier for XE upper bound
LB(:,:,6)=0; UB(:,:,6)=Inf; % Lagrangian multiplier for XT lower bound
LB(:,:,7)=0; UB(:,:,7)=Inf; % Lagrangian multiplier for XT upper bound
% giving start-points
PX0=zeros(K,J,7);
PX0(:,:,1)=2.2; PX0(:,:,2)=0.8; PX0(:,:,3)=0.12;

% === Solve three cases ===
% Case 1
caseindex=1;
disp('Solving case 1: CAFE > Standard (surpassing)');
[PX1,fval,exitflag1]=fmincon(@obj0,PX0,[],[],[],[],LB,UB,@pxfoc,options,caseindex);
disp(['  solved: exitflag = ' num2str(exitflag1)]);
CAFE1=getCAFE(PX1);

% Case 2
caseindex=2;
disp('Solving case 2: CAFE = Standard (binding)');
[PX2,fval,exitflag2]=fmincon(@obj0,PX0,[],[],[],[],LB,UB,@pxfoc,options,caseindex);
disp(['  solved: exitflag = ' num2str(exitflag2)]);
CAFE2=getCAFE(PX2);

% Case 3
caseindex=3;
disp('Solving case 3: CAFE < Standard (violating)');
[PX3,fval,exitflag3]=fmincon(@obj0,PX0,[],[],[],[],LB,UB,@pxfoc,options,caseindex);
disp(['  solved: exitflag = ' num2str(exitflag3)]);
CAFE3=getCAFE(PX3);

disp('---');
% identify which case is valid
if sum(CAFE1>mpgstd)==K && exitflag1 > 0,
    disp(sprintf('Firms surpass the standard [1](CAFE=%-0.2f > Std=%-0.1f)',CAFE1(1),mpgstd));
    PXSOL=PX1;
    CAFESOL=CAFE1;
elseif sum(CAFE3<mpgstd)==K && exitflag3 > 0,
    disp(sprintf('Firms violate the standard [3](CAFE=%-0.2f < Std=%-0.1f)',CAFE3(1),mpgstd));
    PXSOL=PX3;
    CAFESOL=CAFE3;
elseif exitflag2 > 0
    disp(sprintf('Firms bind with the standard [2](CAFE=%-0.2f < Std=%-0.1f)',CAFE2(1),mpgstd));
    PXSOL=PX2;
    CAFESOL=CAFE2;
else
    disp('no solution found.');
end

for k=1:K,
    disp(sprintf('k=%d price=%0.0f xE=%0.4f xT=%0.4f CAFE=%0.2f',k,PXSOL(k,1,1)*1e4,PXSOL(k,1,2),PXSOL(k,1,3),CAFESOL(k)));
end
